-- vim: ts=2 sw=2 ai et

local NAME = "Track-o-Matique"

ToM = LibStub ("AceAddon-3.0"):NewAddon (NAME, "AceConsole-3.0", "AceEvent-3.0")

local options = {
  type = 'group',
  handler = ToM,
  childGroups = 'tab',
  args = {
    options = {
      type = 'group', name = "Options", desc = NAME.." Options", order = 1,
      args = {
        automatic = {
          type = 'toggle', name = "Automatic tracking", order = 1,
          desc = "Automatically change tracking based on current target.",
          get = "GetVal", set = "SetVal"
        },
        debug = {
          type = 'toggle', name = "Debugging", order = -1,
          desc = "Enable printing of debugging information.",
          get = "GetVal", set = "SetVal"
        }
        -- TODO: Add a "default tracking type" for when no target is selected?
      }
    },
    about = {
      type = 'group', name = "About", desc = "About "..NAME, order = -1,
      args = {
        __ = { type = 'header',
               name = NAME..GetAddOnMetadata (NAME,"Version"), order = 1 },
        _1 = { type = 'description',
               name = GetAddOnMetadata (NAME, "Notes"), order = 10 },
        _2 = { type = 'description',
               name = "By "..GetAddOnMetadata (NAME, "Author"), order = 20 },
        _3 = { type = 'description', name = "", order = 30 },
        _4 = { type = 'description',
               name = "Copyright(C) 2008 Johny Mattsson", order = 40 },
        _5 = { type = 'description', name = "", order = 50 },
        _6 = { type = 'description',
               name = "Build "..GetAddOnMetadata (NAME,"X-Build"):match ("%d+"),
               order = 60 }
      }
    }
  }
}

-- Default profile options
local defaults = {
  profile = {
    automatic = true,
    debug = false,
  }
}


function ToM:GetVal (info)
  return self.db.profile[info[#info]]
end

function ToM:SetVal (info, v)
  self.db.profile[info[#info]] = v
end

-- Avoid having to iterate over GetTrackingInfo to pull out the active tracking,
-- use lookup tables based on the current tracking texture instead
ToM.type2texture = {
  Beast = "Interface\\Icons\\Ability_Tracking",
  Demon = "Interface\\Icons\\Spell_Shadow_SummonFelHunter",
  Dragonkin = "Interface\\Icons\\INV_Misc_Head_Dragon_01",
  Elemental = "Interface\\Icons\\Spell_Frost_SummonWaterElemental",
  Giant = "Interface\\Icons\\Ability_Racial_Avatar",
  Humanoid = "Interface\\Icons\\Spell_Holy_PrayerOfHealing",
  Undead = "Interface\\Icons\\Spell_Shadow_DarkSummoning"
}
-- Reverse look up table
ToM.texture2type = {}

function ToM:AutoTrack ()
  local currentTexture = GetTrackingTexture ()
  local targetType = UnitCreatureType ("target")
  if not targetType then
    -- No target
    return
  end
  if self.type2texture[targetType] ~= currentTexture then
    local index = self.type2index[targetType]
    if (index) then
      if self.db.profile.debug then
        self:Print("Activating "..self.type2name[targetType])
      end
      SetTracking (index)
    elseif self.db.profile.debug then
      self:Print ("Target is untrackable! ("..targetType..")")
    end
    return 1
  end
end

function ToM:UpdateLookupTables ()
  local count = 0
  self.type2index = { }
  self.type2name = { }
  for i = 1, GetNumTrackingTypes () do
    local name, texture, active, category = GetTrackingInfo (i)
    local typ = self.texture2type[texture]
    if typ then
      -- Unit type we can and want to track
      self.type2index[typ] = i
      self.type2name[typ] = name
      count = count + 1
    end
  end
  if self.db.profile.debug then
    self:Print ("Now have "..count.." tracking types enabled")
  end
end

-- GetTrackingInfo doesn't work until the character info is loaded
function ToM:PLAYER_ENTERING_WORLD ()
  self:UpdateLookupTables ()
end
-- Might have learned a new tracking type
function ToM:LEARNED_SPELL_IN_TAB (tabId)
  self:UpdateLookupTables ()
end
-- Update tracking, maybe
function ToM:PLAYER_TARGET_CHANGED ()
  if self.db.profile.automatic then
    self:AutoTrack ()
  end
end

function ToM:OnInitialize ()
  local _,playerClass = UnitClass("player")
  if (playerClass ~= "HUNTER") then
    self.noload = true
    return
  end

  for typ, tex in pairs(self.type2texture) do
    self.texture2type[tex] = typ
  end

  LibStub ("AceConfigRegistry-3.0"):RegisterOptionsTable (NAME, options)
  local ACD = LibStub ("AceConfigDialog-3.0")
  ACD:AddToBlizOptions (NAME, GetAddOnMetadata (NAME, "Title"))
  -- Bring up GUI config screen
  self:RegisterChatCommand ("track-o-matique", function ()
    InterfaceOptionsFrame_OpenToCategory (ACD.BlizOptions[NAME].frame) end)

  -- Run auto-track command
  self:RegisterChatCommand ("tom", function() ToM:AutoTrack () end)

  -- Profile db
  self.db = LibStub ("AceDB-3.0"):New ("Track_o_MatiqueDB", defaults)
end

function ToM:OnEnable ()
  if self.noload then
    self:Print("not loaded - this is a hunter-only add-on")
    return
  end
  self:RegisterEvent("LEARNED_SPELL_IN_TAB")
  self:RegisterEvent("PLAYER_ENTERING_WORLD")
  self:RegisterEvent("PLAYER_TARGET_CHANGED")
  if self.db.profile.automatic then
    self:Print ("Automatic tracking now ON")
  else
    self:Print ("Automatic tracking now OFF (use /tom for manual tracking)")
  end
end

